@@ -29,6 +29,10 @@ PARAM_CHANNEL = 'CHANNEL'
 PARAM_META_TYPE = 'META_TYPE'
 PARAM_SETTINGS = 'SETTINGS'
 PARAM_META = 'META'
+PARAM_META_WF = 'META_WF'
+PARAM_META_APP = 'META_APP'
+PARAM_META_SERVICE = 'META_SERVICE'
+PARAM_META_VIEW = 'META_VIEW'
 PARAM_MEDIA_TYPE = 'MEDIA_TYPE'
 
 CONDITION_ACTION_RENDER = 'render'
