@@ -8,7 +8,8 @@ urlpatterns = patterns('',
     (r'^jxAppTemplate/(?P<app>.*)$', 'ximpia.xpcore.views.jxAppTemplate'),
     # Exec Server action
     (r'^(?P<appSlug>[a-zA-Z0-9-]+)/do/(?P<actionSlug>[a-zA-Z0-9-]+)/(?P<actionAttrs>.*)$', 'ximpia.xpcore.views.execActionMsg'),
-    # Show Server View
+    # Show Server View.
+    (r'^(?P<appSlug>[a-zA-Z0-9-]+)/$', 'ximpia.xpcore.views.showView'),
     (r'^(?P<viewSlug>[a-zA-Z0-9-]+)/(?P<viewAttrs>.*)$', 'ximpia.xpcore.views.showView'),
     (r'^(?P<appSlug>[a-zA-Z0-9-]+)/(?P<viewSlug>[a-zA-Z0-9-]+)/(?P<viewAttrs>.*)$', 'ximpia.xpcore.views.showView'),
     # Ajax Urls
