@@ -16,7 +16,7 @@ from django.shortcuts import render_to_response
 from django.utils.translation import ugettext as _
 from django.http import Http404 
 
-from ximpia.xpcore.util import get_class, AttrDict
+from ximpia.xpcore.util import get_class, AttrDict, get_app_full_path, get_app_path
 from models import context, context_view, ctx, JsResultDict
 from service import XpMsgException, view_tmpl, SearchService, TemplateService, CommonService
 from data import ViewDAO, ActionDAO, ApplicationDAO
@@ -327,7 +327,8 @@ def jxDataQuery(request, **args):
         raise XpMsgException(AttributeError, _('app and dbClass must be defined.'))
     dbClass = request.REQUEST['dbClass']
     dbApplication = ApplicationDAO(args['ctx'])
-    app = request.REQUEST['app']
+    #logger.debug('jxDataQuery :: app: {}'.format(request.REQUEST['app'], get_app_full_path(request.REQUEST['app'])))
+    app = get_app_full_path(request.REQUEST['app'])
     application = dbApplication.get(name=app)
     # app: ximpia_site.web, MyDAO => ximpia_site.web.data.MyDAO
     classPath = app + '.data.' + dbClass
@@ -607,16 +608,16 @@ def jxService(request, **args):
         implFields = impl.split('.')
         method = implFields[len(implFields)-1]
         classPath = ".".join(implFields[:-1])
-        logger.debug('classPath: %s' % (classPath) )
+        logger.debug('classPath: %s' % (classPath))
         if method.find('_') == -1 or method.find('__') == -1:
             cls = get_class(classPath)
             obj = cls(args['ctx'])
             super(cls, obj).__init__(args['ctx'])
             obj.request = request
             if (len(viewAttrs) == 0):
-                result = eval('obj.' + method)()
+                result = getattr(obj, method)()
             else:
-                result = eval('obj.' + method)(**viewAttrs)
+                result = getattr(obj, method)(**viewAttrs)
         else:
             logger.debug( 'private methods...' )
             raise Http404
@@ -648,8 +649,11 @@ def jxSave(request, **args):
             # resolve form, set to args['ctx'].form
             logger.debug('jxSave :: form: %s' % (request.REQUEST['form']) )
             formId = request.REQUEST['form']
-            app = request.REQUEST['app']
-            formModule = getattr(getattr(__import__(app.split('.')[0]), app.split('.')[1]), 'forms')
+            app = request.REQUEST['app']            
+            app_path = get_app_full_path(app)
+            logger.debug('formId: {} app: {} app_path: {}'.format(formId, app, app_path))
+            formModule = getattr(getattr(__import__(app_path + '.forms'), app_path.split('.')[1]), 'forms')
+            logger.debug('formModule: {}'.format(formModule))
             classes = dir(formModule)
             resolvedForm = None
             for myClass in classes:
@@ -686,7 +690,8 @@ def jxSave(request, **args):
             obj = CommonService(args['ctx'])
             obj.request = request
             if isFormValid == True:
-                obj._setMainForm(args['ctx'].form)
+            	logger.debug('jxSave :: Form is valid!!!')
+                obj._set_main_form(args['ctx'].form)
                 result = obj.save()
             else:
                 if settings.DEBUG == True:
@@ -774,9 +779,10 @@ def jxDelete(request, **args):
 
 @context_view()
 @view_tmpl()
-def showView(request, appSlug, viewSlug, viewAttrs, **args):
+def showView(request, appSlug='front', viewSlug='home', viewAttrs='', **args):
     """
     Show url view. Application code and view name are parsed from the url.
+    urls not following /appSlug/viewSlug mapped into urls???? appSlug would be default app from settings
 
     **Required Attributes**
 
@@ -814,7 +820,7 @@ def execActionMsg(request, appSlug, actionSlug, actionAttrs, **args):
     """
     Executes an action and shows a message of result of action.
     """
-    logger.debug('appslug: %s actionslug: %s actionAttrs: %s' % (appSlug, actionSlug, actionAttrs) )
+    logger.debug('execActionMsg :: appslug: %s actionslug: %s actionAttrs: %s' % (appSlug, actionSlug, actionAttrs) )
     dbApplication = ApplicationDAO(args['ctx'])
     application = dbApplication.get(slug=appSlug)
     db = ActionDAO(args['ctx'])
@@ -842,6 +848,6 @@ def execActionMsg(request, appSlug, actionSlug, actionAttrs, **args):
         else:
             result = eval('obj.' + method)(*actionAttrTuple)
     else:
-        logger.debug( 'xpcore :: execAction :: private methods...' )
+        logger.debug('xpcore :: execAction :: private methods...')
         raise Http404
     return result
